@@ -73,8 +73,28 @@ void PsychInitializePsychHID(void)
 	}
 
     #if (PSYCH_SYSTEM == PSYCH_OSX) && defined(__LP64__)
-    for (i = 0; i < MAXDEVICEINDEXS; i++)
-        deviceInterfaces[i] = NULL;
+    for (i = 0; i < MAXDEVICEINDEXS; i++) deviceInterfaces[i] = NULL;
+
+    // Try to load all bundles from Psychtoolbox/PsychHardware/
+    // This loads the HID_Utilities.framework bundle if it is present. The whole point of it is
+    // to allow our statically compiled-in version of the library to find the location of
+    // the XML file with the database of (vendorId, productId) -> (VendorName, ProductName) and
+    // (usagePage, usage) -> (usageName) mappings.
+    //
+    // In practice, the XML file only serves as a fallback, and one that doesn't contain much
+    // useful info for mainstream products, only for a few niche products. Given its limited
+    // value, i think we can refrain from shipping the framework as part of Psychtoolbox and
+    // just provide the option to use it (== its XML file) if users decide to install it themselves.
+    char tmpString[1024];
+    
+    sprintf(tmpString, "%sPsychHardware/", PsychRuntimeGetPsychtoolboxRoot(FALSE));
+    CFStringRef urlString = CFStringCreateWithCString(kCFAllocatorDefault, tmpString, kCFStringEncodingASCII);
+    CFURLRef directoryURL = CFURLCreateWithFileSystemPath(kCFAllocatorDefault, urlString, kCFURLPOSIXPathStyle, false);
+    CFRelease(urlString);
+    CFArrayRef bundleArray = CFBundleCreateBundlesFromDirectory(kCFAllocatorDefault, directoryURL, NULL);
+    CFRelease(directoryURL);
+    CFRelease(bundleArray);    
+
     #endif
     
 	// Initialize OS specific interfaces and routines:
@@ -458,28 +478,6 @@ void PsychHIDVerifyInit(void)
     
     // Verify no security sensitive application is blocking our low-level access to HID devices:
 	PsychHIDWarnInputDisabled(NULL);
-    
-    #if defined(__LP64__)
-    // Try to load all bundles from Psychtoolbox/PsychHardware/
-    // This loads the HID_Utilities.framework bundle if it is present. The whole point of it is
-    // to allow our statically compiled-in version of the library to find the location of
-    // the XML file with the database of (vendorId, productId) -> (VendorName, ProductName) and
-    // (usagePage, usage) -> (usageName) mappings.
-    //
-    // In practice, the XML file only serves as a fallback, and one that doesn't contain much
-    // useful info for mainstream products, only for a few niche products. Given its limited
-    // value, i think we can refrain from shipping the framework as part of Psychtoolbox and
-    // just provide the option to use it (== its XML file) if users decide to install it themselves.
-    char tmpString[1024];
-    
-    sprintf(tmpString, "%sPsychHardware/", PsychRuntimeGetPsychtoolboxRoot(FALSE));
-    CFStringRef urlString = CFStringCreateWithCString(kCFAllocatorDefault, tmpString, kCFStringEncodingASCII);
-    CFURLRef directoryURL = CFURLCreateWithFileSystemPath(kCFAllocatorDefault, urlString, kCFURLPOSIXPathStyle, false);
-    CFRelease(urlString);
-    CFArrayRef bundleArray = CFBundleCreateBundlesFromDirectory(kCFAllocatorDefault, directoryURL, NULL);
-    CFRelease(directoryURL);
-    CFRelease(bundleArray);    
-    #endif
 }
 
 /*
@@ -520,6 +518,7 @@ static IOHIDDeviceInterface122** HIDCreateOpenDeviceInterface(pRecDevice pDevice
     IOCFPlugInInterface** ppPlugInInterface = NULL;
     IOHIDDeviceInterface** interface = NULL;
     io_service_t hidDevice = 0L;
+    mach_port_t port;
     
     // Get low-level device for given HIDDeviceRef from HID Utilities v2.0:
     hidDevice = AllocateHIDObjectFromIOHIDDeviceRef((IOHIDDeviceRef) pDevice);
@@ -532,23 +531,35 @@ static IOHIDDeviceInterface122** HIDCreateOpenDeviceInterface(pRecDevice pDevice
     // Create and open interface for IORegistry device:
     result = IOCreatePlugInInterfaceForService (hidDevice, kIOHIDDeviceUserClientTypeID,
                                                 kIOCFPlugInInterfaceID, &ppPlugInInterface, &score);
-    if (kIOReturnSuccess == result)
+    if ((kIOReturnSuccess == result) && ppPlugInInterface)
     {
         // Call a method of the intermediate plug-in to create the device interface
         plugInResult = (*ppPlugInInterface)->QueryInterface (ppPlugInInterface,
                                                              CFUUIDGetUUIDBytes (kIOHIDDeviceInterfaceID), (void *) &interface);
-        if (S_OK != plugInResult)
-            printf("PTB-ERROR: PsychHID: Couldnt query HID class device interface from plugInInterface: %x [].", plugInResult);
+        if ((S_OK != plugInResult) || (NULL == interface))
+            printf("PTB-ERROR: PsychHID: Couldn't query HID class device interface from plugInInterface: %x.\n", plugInResult);
+
+        // Release ppPlugInInterface in any case - no longer needed:
         IODestroyPlugInInterface(ppPlugInInterface);
     }
     else
-        printf("PTB-ERROR: PsychHID: Failed to create **plugInInterface via IOCreatePlugInInterfaceForService: %x [].\n", result);
+        printf("PTB-ERROR: PsychHID: Failed to create **plugInInterface via IOCreatePlugInInterfaceForService: %x.\n", result);
     
+	// Successfully retrieved an interface?
 	if (NULL != interface)
 	{
+		// Yes: Open it.
 		result = (*interface)->open(interface, 0);
-		if (kIOReturnSuccess != result)
-			printf("PTB-ERROR: PsychHID: Failed to open HID device low-level interface via open: %x [].\n", result);
+		if (kIOReturnSuccess != result) {
+			printf("PTB-ERROR: PsychHID: Failed to open HID device low-level interface via open: %x.\n", result);
+		}
+		else {
+			// Interface open. Create async mach port for it to deliver interrupt in endpoint data:
+			result = (*interface)->createAsyncPort(interface, &port);
+			if (kIOReturnSuccess != result) {
+				printf("PTB-ERROR: PsychHID: Failed to create async port for HID device low-level interface: %x.\n", result);
+			}
+		}
 	}
     
     // Release reference to high-level device via HID Utilities v2.0::
