@@ -83,7 +83,7 @@ PsychError PsychSynchronizeDisplayScreens(int *numScreens, int* screenIds, int*
 	
 	if (numScreens == NULL) PsychErrorExitMsg(PsychError_internal, "NULL-Ptr passed as numScreens argument!");
 	if (*numScreens < 0 || *numScreens > PsychGetNumDisplays()) PsychErrorExitMsg(PsychError_internal, "Invalid number passed as numScreens argument! (Negative or more than available screens)");
-	if (syncMethod < 0 || syncMethod > 2) PsychErrorExitMsg(PsychError_internal, "Invalid syncMethod argument passed!");
+	if (syncMethod > 2) PsychErrorExitMsg(PsychError_internal, "Invalid syncMethod argument passed!");
 	if (syncTimeOut < 0) PsychErrorExitMsg(PsychError_internal, "Invalid (negative) syncTimeOut argument passed!");
 	if (allowedResidual < 0) PsychErrorExitMsg(PsychError_internal, "Invalid (negative) allowedResidual argument passed!");
 	
@@ -663,63 +663,68 @@ psych_bool PsychWaitForBufferswapPendingOrFinished(PsychWindowRecordType* window
 unsigned int PsychGetNVidiaGPUType(PsychWindowRecordType* windowRecord)
 {
 #if PSYCH_SYSTEM == PSYCH_OSX || PSYCH_SYSTEM == PSYCH_LINUX
-	psych_uint32 chipset, card_type;
+    psych_uint32 chipset, card_type;
 
-	// Get hardware id code from gpu register:
-	psych_uint32 reg0 = PsychOSKDReadRegister((windowRecord) ? windowRecord->screenNumber : 0, NV03_PMC_BOOT_0, NULL);
-	
-	/* We're dealing with >=NV10 */
-	if ((reg0 & 0x0f000000) > 0) {
-		/* Bit 27-20 contain the architecture in hex */
-		chipset = (reg0 & 0xff00000) >> 20;
-		/* NV04 or NV05 */
-	} else if ((reg0 & 0xff00fff0) == 0x20004000) {
-		if (reg0 & 0x00f00000)
-			chipset = 0x05;
-		else
-			chipset = 0x04;
-	} else
-		chipset = 0xff;
-	
-	switch (chipset & 0xf0) {
-		case 0x00:
-			// NV_04/05: RivaTNT , RivaTNT2
-			card_type = 0x04;
-			break;
-		case 0x10:
-		case 0x20:
-		case 0x30:
-			// NV30 or earlier: GeForce-5 / GeForceFX and earlier:
-			card_type = chipset & 0xf0;
-			break;
-		case 0x40:
-		case 0x60:
-			// NV40: GeForce6/7 series:
-			card_type = 0x40;
-			break;
-		case 0x50:
-		case 0x80:
-		case 0x90:
-		case 0xa0:
-			// NV50: GeForce8/9/G100-G300.
-			card_type = 0x50;
-			break;
-		case 0xc0:
-			// Fermi: GeForce G400/500 series:
-			card_type = 0xc0;
-			break;
-		case 0xe0:
-			// Kepler: GeForce G600+ series:
-			card_type = 0xe0;
-			break;
-		default:
-			printf("PTB-DEBUG: Unknown NVidia chipset 0x%08x \n", reg0);
-			card_type = 0x00;
-	}
-	
-	return(card_type);
+    // Get hardware id code from gpu register:
+    psych_uint32 reg0 = PsychOSKDReadRegister((windowRecord) ? windowRecord->screenNumber : 0, NV03_PMC_BOOT_0, NULL);
+
+    // Special case: Register read delivers "hardware powered down" value, because this
+    // GPU is offline, e.g., in a hybrid-graphics setup. Return a special code to signal
+    // to caller that it doesn't need to bother anymore with this GPU:
+    if (reg0 == 0xffffffff) return(0xffffffff);
+
+    /* We're dealing with >=NV10 */
+    if ((reg0 & 0x0f000000) > 0) {
+        /* Bit 27-20 contain the architecture in hex */
+        chipset = (reg0 & 0xff00000) >> 20;
+        /* NV04 or NV05 */
+    } else if ((reg0 & 0xff00fff0) == 0x20004000) {
+        if (reg0 & 0x00f00000)
+            chipset = 0x05;
+        else
+            chipset = 0x04;
+    } else
+        chipset = 0xff;
+
+    switch (chipset & 0xf0) {
+        case 0x00:
+            // NV_04/05: RivaTNT , RivaTNT2
+            card_type = 0x04;
+            break;
+        case 0x10:
+        case 0x20:
+        case 0x30:
+            // NV30 or earlier: GeForce-5 / GeForceFX and earlier:
+            card_type = chipset & 0xf0;
+            break;
+        case 0x40:
+        case 0x60:
+            // NV40: GeForce6/7 series:
+            card_type = 0x40;
+            break;
+        case 0x50:
+        case 0x80:
+        case 0x90:
+        case 0xa0:
+            // NV50: GeForce8/9/G100-G300.
+            card_type = 0x50;
+            break;
+        case 0xc0:
+            // Fermi: GeForce G400/500 series:
+            card_type = 0xc0;
+            break;
+        case 0xe0:
+            // Kepler: GeForce G600+ series:
+            card_type = 0xe0;
+            break;
+        default:
+            printf("PTB-DEBUG: Unknown NVidia chipset 0x%08x \n", reg0);
+            card_type = 0x00;
+    }
+
+    return(card_type);
 #else
-	return(0);
+    return(0);
 #endif
 }
 
