@@ -194,8 +194,13 @@ PsychError SCREENOpenWindow(void)
 		PsychGetGlobalScreenRect(screenNumber, screenrect);
 		if (PsychMatchRect(screenrect, rect)) useAGL=FALSE;
 
-		// Override for use on f$%#$Fd OS/X 10.5.3 - 10.5.6 with NVidia GF 8800 GPU's:
-		if (PsychPrefStateGet_ConserveVRAM() & kPsychUseAGLCompositorForFullscreenWindows) useAGL = TRUE;
+		// Override for use with Quartz compositor: Must not capture/release screen, therefore
+        // set useAGL = true to prevent screen capture/release:
+		if ((PsychPrefStateGet_ConserveVRAM() & kPsychUseAGLCompositorForFullscreenWindows) ||
+            (PsychPrefStateGet_WindowShieldingLevel() < 2000) ||
+            (PsychPrefStateGet_ConserveVRAM() & kPsychUseAGLForFullscreenWindows)) {
+            useAGL = TRUE;
+        }
 	}
 	else {
 		// Non OS/X system: Do not use AGL ;-)
@@ -224,7 +229,7 @@ PsychError SCREENOpenWindow(void)
 	
 	specialflags=0;
     PsychCopyInIntegerArg(9,FALSE,&specialflags);
-    if (specialflags < 0 || (specialflags > 0 && specialflags!=kPsychGUIWindow)) PsychErrorExitMsg(PsychError_user, "Invalid 'specialflags' provided.");
+    if (specialflags < 0 || (specialflags > 0 && !(specialflags & (kPsychGUIWindow | kPsychGUIWindowWMPositioned)))) PsychErrorExitMsg(PsychError_user, "Invalid 'specialflags' provided.");
 
     // Optional clientRect defined? If so, we need to enable our internal panel scaler and
     // the imaging pipeline to actually use the scaler:
@@ -559,6 +564,10 @@ PsychError SCREENOpenWindow(void)
         windowRecord->panelFitterParams[5] = 0; // dstY0
         windowRecord->panelFitterParams[6] = (int) windowRecord->fboTable[windowRecord->inputBufferFBO[0]]->width;  // dstX1
         windowRecord->panelFitterParams[7] = (int) windowRecord->fboTable[windowRecord->inputBufferFBO[0]]->height; // dstY1
+        
+        windowRecord->panelFitterParams[8] = 0; // rotation angle.
+        windowRecord->panelFitterParams[9] = windowRecord->panelFitterParams[6]/2; // rotation center X.
+        windowRecord->panelFitterParams[10]= windowRecord->panelFitterParams[7]/2; // rotation center Y.
     }
 
 	// On OS-X, if we are in quad-buffered frame sequential stereo mode, we automatically generate
@@ -606,6 +615,32 @@ PsychError SCREENOpenWindow(void)
 
     PsychTestForGLErrors();
 
+    // Homegrown frame-sequential strereo mode on a EGL backed window active?
+    if ((windowRecord->stereomode == kPsychFrameSequentialStereo) && (windowRecord->specialflags & kPsychIsEGLWindow)) {
+        // Detach the OpenGL context from window surface. The following PsychSetDrawingTarget()
+        // command will rebind the context as a first step, but it will not attach it to the
+        // windowing system framebuffer surface (== the associated EGLSurface) anymore due to
+        // the selected frame-sequential stereo mode on EGL. This will allow the background
+        // frame-sequential stereo swapper-thread to bind its context to the surface and all
+        // will be good. Rationale: With a EGL windowing system backend, only at most one context
+        // is allowed to attach to a EGL surface (window framebuffer) at a given time. Because the
+        // stereo thread needs to bind its context permanently to the surface to do its job, we
+        // must make sure going forward that we'll never ever bind our contexts from the master-thread
+        // to 'windowRecord's surface again, or bad things will happen:
+        PsychSetDrawingTarget((PsychWindowRecordType*) 0x1);
+        PsychOSUnsetGLContext(windowRecord);
+        windowRecord->specialflags |= kPsychSurfacelessContexts;
+        PsychSetDrawingTarget(windowRecord);
+        if (PsychPrefStateGet_Verbosity() > 4) printf("PTB-INFO: Preventing master-thread contexts from future binding to this windows (%i) backing surface.\n", windowRecord->windowIndex);
+
+        // TODO: Ideally we should check if KHR_surfaceless_context extension is supported,
+        // because otherwise this won't work and we should reject use of multi-threaded ops
+        // like frame-sequential stereo or async flips as unsupported on a given system.
+        // Doing this check is a bit difficult at the moment without querying the EGL
+        // extension string - we don't want to introduce a hard dependency on libEGL at
+        // this point of development...
+    }
+
     // Reset flipcounter and missed flip deadline counter to zero:
     windowRecord->flipCount = 0;
     windowRecord->nr_missed_deadlines = 0;
@@ -631,15 +666,18 @@ PsychError SCREENPanelFitter(void)
         "user to make sure source and destination framebuffer have already the same aspect ratio.\n"
         "This function allows to define new src and dst rectangles, thereby implicitely defining scaling "
         "and filtering properties. It optionally takes new settings in 'newParams' and returns old settings "
-        "in 'oldParams'. The parameters are 8-element vectors of format\n"
-        "params = [srcX0, srcY0, srcX1, srcY1, dstX0, dstY0, dstX1, dstY1];\n"
+        "in 'oldParams'. The function also allows to define a rotation angle for rotation of the output image. "
+        "The parameters are 11-element vectors of format\n"
+        "params = [srcX0, srcY0, srcX1, srcY1, dstX0, dstY0, dstX1, dstY1, angle, rotCX, rotCY];\n"
         "These tuples define top-left and bottom-right (x,y) corners of the source and destination "
-        "rectangles for the (scaled)blit.\n"
+        "rectangles for the (scaled)blit, and the rotation 'angle' if display rotation is requested. "
+        "The angle and rotCX and rotCY parameters are optional and assumed to be zero if omitted, ie., "
+        "no rotation. rotCX and rotCY define the center of rotation if a rotation is requested.\n"
         "You usually won't call this function directly, but leave the job to a higher-level setup "
         "routine, e.g., PsychImaging() and its 'UsePanelFitter' setup code.\n\n";
     static char seeAlsoString1[] = "OpenWindow";
 
-    PsychWindowRecordType   *windowRecord;    
+    PsychWindowRecordType   *windowRecord;
     double* outParams;
     int*    newParams;
     int     count, i;
@@ -656,14 +694,25 @@ PsychError SCREENPanelFitter(void)
     PsychAllocInWindowRecordArg(1, TRUE, &windowRecord);
     
     // Return optional fitter settings:
-    PsychAllocOutDoubleMatArg(1, FALSE, 1, 8, 1, &outParams);
-    for (i = 0; i < 8; i++) outParams[i] = (double) windowRecord->panelFitterParams[i];
+    PsychAllocOutDoubleMatArg(1, FALSE, 1, 11, 1, &outParams);
+    for (i = 0; i < 11; i++) outParams[i] = (double) windowRecord->panelFitterParams[i];
     
     // Get optional new panelFitter settings:
     if (PsychAllocInIntegerListArg(2, FALSE, &count, &newParams)) {
-        if (count != 8) PsychErrorExitMsg(PsychError_user, "'newParams' must be a vector with 8 integer elements.");
+        if ((count < 8) || (count > 11)) PsychErrorExitMsg(PsychError_user, "'newParams' must be a vector with 8 to 11 integer elements.");
         for (i = 0; i < count; i++) windowRecord->panelFitterParams[i] = newParams[i];
+
+        // Fallback path needed (due to lack of FBO blit or non-zero rotation angle) and problematic new config setting?
+        if ((!(windowRecord->gfxcaps & kPsychGfxCapFBOBlit) || (windowRecord->panelFitterParams[8] != 0)) && (PsychPrefStateGet_Verbosity() > 2) &&
+            (windowRecord->panelFitterParams[0] != 0 || windowRecord->panelFitterParams[1] != 0 ||
+             windowRecord->panelFitterParams[2] != (int) PsychGetWidthFromRect(windowRecord->clientrect) ||
+             windowRecord->panelFitterParams[3] != (int) PsychGetHeightFromRect(windowRecord->clientrect))) {
+            // Fallback path for panelFitter in use and sourceRegion is not == full clientRect. This is an
+            // unsupported setting with the fallback, which will cause wrong results. Warn user:
+            printf("PTB-INFO: Non-default 'srcRegion' in call to Screen('PanelFitter') ignored. This is not supported when the\n");
+            printf("PTB-INFO: fallback path or display rotation for the panel fitter is in use.\n");
+        }
     }
-    
+
     return(PsychError_none);
 }
