@@ -60,18 +60,19 @@ void PsychHIDInitializeHIDStandardInterfaces(void)
 	memset(&psychHIDKbQueueActive[0], 0, sizeof(psychHIDKbQueueActive));
 	memset(&psychHIDKbQueueScanKeys[0], 0, sizeof(psychHIDKbQueueScanKeys));
 
-	// We must initialize XLib for multi-threaded operations / access on first
-	// call:
-	// TODO FIXME: We can only do this on Octave for now, not on Matlab!
-	// Matlab uses XLib long before we get a chance to get here, but XInitThreads()
-	// must be called as very first XLib function after process startup or bad things
-	// will happen! So, we can't call it...
-	// Because some system configurations can't handle multi-threaded x at all,
-	// we allow users to opt-out of this if they define an environment variable
-	// PSYCHTOOLBOX_SINGLETHREADEDX.
-	#ifdef PTBOCTAVE3MEX
-	if (NULL == getenv("PSYCHTOOLBOX_SINGLETHREADEDX")) XInitThreads();
-	#endif
+    // We must initialize XLib for multithreading-safe operations / access on first
+    // call if usercode explicitely requests this via environment variable PSYCH_XINITTHREADS.
+    //
+    // We can only do this as opt-in, as XInitThreads() must be called as very first
+    // XLib function after process startup or bad things will happen! We don't have control
+    // over this wrt. Matlab or Octave (especially future Octave 3.7+ with its QT based GUI),
+    // so we implemented our own locking in Screen() and don't need it in PsychHID, as PsychHID's
+    // x-connection is exclusively used by PsychHID's Xinput processing thread. However, there
+    // may be some cases when our own locking is insufficient, due to deficiencies in the
+    // DRI2 XOrg FOSS Mesa graphics driver stack, so some users may want to opt-into use
+    // XLib's threading protection as a work-around if they can guarantee Octave or Matlab
+    // hasn't called any XLib calls already during its running session:
+    if (getenv("PSYCH_XINITTHREADS")) XInitThreads();
 
 	// Open our own private X-Display connection for HID handling:
 	dpy = XOpenDisplay(NULL);
@@ -867,14 +868,14 @@ void PsychHIDOSKbQueueStop(int deviceIndex)
 	event.x_root = 1;
 	event.y_root = 1;
 	event.same_screen = TRUE;
-	event.type = KeyPress;
+	event.type = KeyRelease;
 	event.keycode = 0;
 	event.state = 0;
 	
 	// printf("DEBUG: Sending termination fake event...\n"); fflush(NULL);
-	XSelectInput(event.display, event.window, KeyPressMask);
+	XSelectInput(event.display, event.window, KeyReleaseMask);
 	XFlush(thread_dpy);
-	XSendEvent(event.display, event.window, TRUE, KeyPressMask, (XEvent *) &event);
+	XSendEvent(event.display, event.window, TRUE, KeyReleaseMask, (XEvent *) &event);
 	XFlush(thread_dpy);
 	// printf("DEBUG: DONE.\n"); fflush(NULL);
 
