@@ -1,7 +1,7 @@
 function DownloadPsychtoolbox(targetdirectory, flavor, targetRevision)
 % DownloadPsychtoolbox([targetdirectory][, flavor][, targetRevision])
 %
-% This script downloads the latest Mac OSX, GNU/Linux or Windows
+% This script downloads the latest GNU/Linux, Mac OSX, or Windows
 % Psychtoolbox-3, version 3.0.10 or later, from our git-server to your
 % disk, creating your working copy, ready to use as a new toolbox in your
 % MATLAB/OCTAVE application. Subject to your permission, any old
@@ -9,12 +9,21 @@ function DownloadPsychtoolbox(targetdirectory, flavor, targetRevision)
 % program, checking for all required resources and privileges before it
 % starts.
 %
-% CAUTION: Psychtoolbox *will not work* with GNU/Octave on Microsoft, or
-% with 32-Bit Octave on OSX, as support for these setups has been cancelled
-% for the 3.0.10 series.
+% Note: If you use a Debian derives Linux distribution, e.g., Debian or
+% Ubuntu, consider installing the package octave-psychtoolbox-3 or
+% matlab-psychtoolbox-3 instead from http://neuro.debian.net - This is more
+% convenient ans will provide you with automatic updates.
 %
-% If you want to download older versions of Psychtoolbox than 3.0.10, use
-% the DownloadLegacyPsychtoolbox() function instead of this function.
+% CAUTION: Psychtoolbox *will not work* with GNU/Octave on MS-Windows, or
+% with 32-Bit Octave on OSX, as support for these setups has been cancelled
+% for the 3.0.10 series. It will also not work with 32-Bit Matlab on OSX,
+% or with OSX versions earlier than 10.6.8 "Snow Leopard", unless you
+% choose the unsupported legacy flavor "Psychtoolbox-3.0.10" via the
+% optional 'flavor' parameter.
+%
+% If you want to download older versions of Psychtoolbox than 3.0.10, e.g.,
+% version 3.0.9, use the DownloadLegacyPsychtoolbox() function instead of
+% this function.
 %
 % On Mac OSX, all parameters are optional. On MS-Windows and GNU/Linux, the
 % first parameter "targetdirectory" with the path to the installation
@@ -62,17 +71,16 @@ function DownloadPsychtoolbox(targetdirectory, flavor, targetRevision)
 % instructions. If in doubt, follow instructions on the Wiki!
 %
 % 1. If you don't already have it, you must install the Subversion client.
-% For Mac OSX 10.4, download the latest Mac OSX Subversion client from:
-% web http://metissian.com/projects/macosx/subversion/
-% (You can ignore the Subversion README file. If you do read it, you can
-% skip the instruction to manually add /usr/local/bin to your unix path.
-% That's tricky to do, and not needed for installation and updates because
-% we always specify the full path.) Please note that OS/X 10.5 "Leopard"
-% and later already have Subversion preinstalled, so you may be able to
-% skip step 1.
+% For Mac OSX 10.6 and later, download the latest Mac OSX Subversion client
+% from: http://www.wandisco.com/subversion/download#osx
+% If you have the XCode command line tools installed, you won't need to
+% install subversion as it is included in these tools.
+% 
+% For Windows, download the Windows Subversion client from one of these:
+%
+% http://subversion.apache.org/packages.html#windows
+% http://www.wandisco.com/subversion/download#windows
 %
-% For Windows, download the Windows Subversion client from:
-% web http://subversion.apache.org/packages.html#windows
 % Install the Subversion client on your machine by double-clicking the
 % installer and following the instructions. After installation of the
 % Subversion client, you will need to exit and restart Matlab or Octave, so
@@ -91,6 +99,10 @@ function DownloadPsychtoolbox(targetdirectory, flavor, targetRevision)
 % addpath('D:\MyOwnFolder\Subversion\'). Our installer should find the
 % client then.
 %
+% For Linux, just install the subversion package from your package
+% management tool.
+%
+%
 % 2. On MacOS/X, to install the Psychtoolbox in the default location
 % (/Applications or, failing that, /Users/Shared). Just type:
 %
@@ -145,12 +157,6 @@ function DownloadPsychtoolbox(targetdirectory, flavor, targetRevision)
 % few users have "administrator" privileges, and many don't. By default,
 % writing to the /Applications folder requires administrator privileges.
 %
-% Thus all OSX installers routinely demand an extra authorization (if
-% needed), asking the user to type in the name and password of an
-% administrator before proceeding. We haven't yet figured out how to do
-% that, but we want to offer that option. This conforms to normal
-% installation of an application under Mac OS X.
-%
 % DownloadPsychtoolbox creates the Psychtoolbox folder with permissions set
 % to allow writing by everyone. Our hope is that this will allow updating
 % (by UpdatePsychtoolbox) without need for administrator privileges.
@@ -273,6 +279,11 @@ function DownloadPsychtoolbox(targetdirectory, flavor, targetRevision)
 %
 % 09/14/12 mk  Drop support for Octave on MS-Windows.
 % 09/14/12 mk  Drop support for 32-Bit Octave on OSX.
+% 03/10/13 mk  Add additional svn search-pathes matching UpdatePsychtoolbox et al.
+%              Also update download URL for OSX Suversion client.
+% 07/02/13 mk  Drop support for 32-Bit Matlab on OSX, and thereby for 32-Bit OSX.
+% 07/02/13 mk  Reenable write access for all to Psychtoolbox folder.
+% 07/23/13 mk  Do not prevent execution on 32-Bit Matlab on OSX!
 
 % Flush all MEX files: This is needed at least on M$-Windows for SVN to
 % work if Screen et al. are still loaded.
@@ -280,10 +291,11 @@ clear mex
 
 % Check if this is 32-Bit Octave on OSX, which we don't support anymore:
 if ~isempty(strfind(computer, 'apple-darwin')) && isempty(strfind(computer, '64'))
-    fprintf('Psychtoolbox 3.0.10 and later does no longer work with 32-Bit GNU/Octave on OSX.\n');
-    fprintf('You need to upgrade to a 64-Bit version of Octave on OSX, which is fully supported.\n');
+    fprintf('Psychtoolbox 3.0.11 and later do no longer work with 32-Bit versions of Octave or Matlab on OSX.\n');
+    fprintf('You need to upgrade to a 64-Bit version of Octave or Matlab on OSX, which is fully supported.\n');
     fprintf('You can also use the alternate download function DownloadLegacyPsychtoolbox() to download\n');
-    fprintf('an old legacy copy of Psychtoolbox-3.0.9, which did support 32-Bit Octave 3.2 on OSX.\n');
+    fprintf('an old legacy copy of Psychtoolbox-3.0.9, which did support 32-Bit Octave 3.2 on OSX, or use\n');
+    fprintf('DownloadPsychtoolbox() with flavor ''Psychtoolbox-3.0.10'', which does support 32-Bit Matlab on OSX.\n');
     error('Tried to setup on 32-Bit Octave, which is no longer supported on OSX.');
 end
 
@@ -307,10 +319,10 @@ end
 
 % Check OS
 IsWin = ~isempty(strfind(computer, 'PCWIN')) || strcmp(computer, 'i686-pc-mingw32');
-IsOSX = ~isempty(strfind(computer, 'MAC')) | ~isempty(strfind(computer, 'apple-darwin'));
-IsLinux = strcmp(computer,'GLNX86') | strcmp(computer,'GLNXA64') | ~isempty(strfind(computer, 'linux-gnu'));
+IsOSX = ~isempty(strfind(computer, 'MAC')) || ~isempty(strfind(computer, 'apple-darwin'));
+IsLinux = strcmp(computer,'GLNX86') || strcmp(computer,'GLNXA64') || ~isempty(strfind(computer, 'linux-gnu'));
 
-if ~IsWin & ~IsOSX & ~IsLinux
+if ~IsWin && ~IsOSX && ~IsLinux
     os = computer;
     if strcmp(os,'MAC2')
         os = 'Mac OS9';
@@ -327,7 +339,7 @@ end
 v = ver('matlab');
 if ~isempty(v)
     v = v(1).Version; v = sscanf(v, '%i.%i.%i');
-    if (v(1) < 7) | ((v(1) == 7) & (v(2) < 4)) %#ok<AND2,OR2>
+    if (v(1) < 7) || ((v(1) == 7) && (v(2) < 4))
         % Matlab version < 7.4 detected. This is no longer
         % supported by current PTB beta. Redirect to the last
         % functional PTB for such ancient Matlab's:
@@ -470,24 +482,38 @@ else
     % simply have to hope that it is in some system dependent search path.
 
     % Currently, we only know how to check this for Mac OSX.
-    if IsOSX
-        % Try OS/X 10.5 Leopard install location for svn first:
-        svnpath='/usr/bin/';
-        if exist('/usr/bin/svn','file')~=2
-            % This would have been the default install location of the svn
-            % client bundled with OS/X 10.5.x Leopard. Let's try the
-            % default install location from the web installer:
-            svnpath='/usr/local/bin/';
-            if exist('/usr/local/bin/svn','file')~=2
-                fprintf('The Subversion client "svn" is not in its expected\n');
-                fprintf('location "/usr/local/bin/svn" on your disk. Please \n');
-                fprintf('download and install the most recent Subversion client from:\n');
-                fprintf('web http://metissian.com/projects/macosx/subversion/ -browser\n');
-                fprintf('and then run %s again.\n',mfilename);
-                error('Subversion client is missing. Please install it.');
-            end
+	if IsOSX
+		svnpath = '';
+		
+		if isempty(svnpath) && exist('/opt/subversion/bin/svn', 'file')
+			svnpath = '/opt/subversion/bin/';
+		end
+
+		if isempty(svnpath) && exist('/usr/bin/svn','file')
+			svnpath='/usr/bin/';
+		end
+
+		if isempty(svnpath) && exist('/usr/local/bin/svn','file')
+			svnpath='/usr/local/bin/';
+		end
+
+		if isempty(svnpath) && exist('/bin/svn','file')
+			svnpath='/bin/';
+		end
+
+		if isempty(svnpath) && exist('/opt/local/bin/svn', 'file')
+			svnpath = '/opt/local/bin/';
+		end
+
+        if isempty(svnpath)
+            fprintf('The Subversion client "svn" is not in one of its expected\n');
+            fprintf('locations for Mac OSX  on your disk. Please download and\n');
+            fprintf('install the most recent Subversion client from:\n');
+            fprintf('web http://subversion.apache.org/packages.html#osx -browser\n');
+            fprintf('and then run %s again.\n', mfilename);
+            error('Subversion client is missing. Please install it.');
         end
-    end
+	end
 end
 
 if ~isempty(svnpath)
@@ -575,7 +601,7 @@ fprintf('Good. Your privileges suffice for the requested installation into folde
 
 % Delete old Psychtoolbox
 skipdelete = 0;
-while (exist('Psychtoolbox','dir') | exist(fullfile(targetdirectory,'Psychtoolbox'),'dir')) & (skipdelete == 0)
+while (exist('Psychtoolbox','dir') || exist(fullfile(targetdirectory,'Psychtoolbox'),'dir')) && (skipdelete == 0)
     fprintf('Hmm. You already have an old Psychtoolbox folder:\n');
     p=fullfile(targetdirectory,'Psychtoolbox');
     if ~exist(p,'dir')
@@ -709,7 +735,7 @@ fprintf('%s\n',checkoutcommand);
 fprintf('Downloading. It''s nearly 100 MB, which can take many minutes. \nAlas there may be no output to this window to indicate progress until the download is complete. \nPlease be patient ...\n');
 fprintf('If you see some message asking something like "accept certificate (p)ermanently, (t)emporarily? etc."\n');
 fprintf('then please press the p key on your keyboard, possibly followed by pressing the ENTER key.\n\n');
-if IsOSX | IsLinux
+if IsOSX || IsLinux
     [err]=system(checkoutcommand);
     result = 'For reason, see output above.';
 else
@@ -719,7 +745,7 @@ end
 % MK: Pointless fallbacks disabled, as GitHub only supports https protocol, so a
 % failure of the first try is game over.
 %
-% if err & (downloadmethod < 1)
+% if err && (downloadmethod < 1)
 %     % Failed! Let's retry it via http protocol. This may work-around overly
 %     % restrictive firewalls or otherwise screwed network proxies:
 %     fprintf('Command "CHECKOUT" failed with error code %d: \n',err);
@@ -728,7 +754,7 @@ end
 %     checkoutcommand=[svnpath 'svn checkout ' targetRevision ' http://github.com/Psychtoolbox-3/Psychtoolbox-3/' dflavor '/Psychtoolbox/ ' pt];
 %     fprintf('The following alternative CHECKOUT command asks the Subversion client to \ndownload the Psychtoolbox:\n');
 %     fprintf('%s\n\n',checkoutcommand);
-%     if IsOSX | IsLinux
+%     if IsOSX || IsLinux
 %         [err]=system(checkoutcommand);
 %         result = 'For reason, see output above.';
 %     else
@@ -736,7 +762,7 @@ end
 %     end    
 % end
 % 
-% if err & (downloadmethod > 0)
+% if err && (downloadmethod > 0)
 %     % Failed! Let's retry it via https protocol. This may work-around overly
 %     % restrictive firewalls or otherwise screwed network proxies:
 %     fprintf('Command "CHECKOUT" failed with error code %d: \n',err);
@@ -745,7 +771,7 @@ end
 %     checkoutcommand=[svnpath 'svn checkout ' targetRevision ' https://github.com/Psychtoolbox-3/Psychtoolbox-3/' dflavor '/Psychtoolbox/ ' pt];
 %     fprintf('The following alternative CHECKOUT command asks the Subversion client to \ndownload the Psychtoolbox:\n');
 %     fprintf('%s\n\n',checkoutcommand);
-%     if IsOSX | IsLinux
+%     if IsOSX || IsLinux
 %         [err]=system(checkoutcommand);
 %         result = 'For reason, see output above.';
 %     else
@@ -788,27 +814,27 @@ else
     fprintf('Success.\n\n');
 end
 
-% fprintf(['Now setting permissions to allow everyone to write to the Psychtoolbox folder. This will \n'...
-%     'allow future updates by every user on this machine without requiring administrator privileges.\n']);
-% try
-%     if IsOSX | IsLinux
-%         [s,m]=fileattrib(p,'+w','a','s'); % recursively add write privileges for all users.
-%     else
-%         [s,m]=fileattrib(p,'+w','','s'); % recursively add write privileges for all users.
-%     end
-% catch
-%     s = 0;
-%     m = 'Setting file attributes is not supported under Octave.';
-% end
-% 
-% if s
-%     fprintf('Success.\n\n');
-% else
-%     fprintf('\nFILEATTRIB failed. Psychtoolbox will still work properly for you and other users, but only you\n');
-%     fprintf('or the system administrator will be able to run the UpdatePsychtoolbox script to update Psychtoolbox,\n');
-%     fprintf('unless you or the system administrator manually set proper write permissions on the Psychtoolbox folder.\n');
-%     fprintf('The error message of FILEATTRIB was: %s\n\n', m);
-% end
+fprintf(['Now setting permissions to allow everyone to write to the Psychtoolbox folder. This will \n'...
+    'allow future updates by every user on this machine without requiring administrator privileges.\n']);
+try
+    if IsOSX || IsLinux
+        [s,m]=fileattrib(p,'+w','a','s'); % recursively add write privileges for all users.
+    else
+        [s,m]=fileattrib(p,'+w','','s'); % recursively add write privileges for all users.
+    end
+catch
+    s = 0;
+    m = 'Setting file attributes is not supported under Octave.';
+end
+
+if s
+    fprintf('Success.\n\n');
+else
+    fprintf('\nFILEATTRIB failed. Psychtoolbox will still work properly for you and other users, but only you\n');
+    fprintf('or the system administrator will be able to run the UpdatePsychtoolbox script to update Psychtoolbox,\n');
+    fprintf('unless you or the system administrator manually set proper write permissions on the Psychtoolbox folder.\n');
+    fprintf('The error message of FILEATTRIB was: %s\n\n', m);
+end
 
 fprintf('You can now use your newly installed ''%s''-flavor Psychtoolbox. Enjoy!\n',flavor);
 fprintf('Whenever you want to upgrade your Psychtoolbox to the latest ''%s'' version, just\n',flavor);
