@@ -1,4 +1,4 @@
-function varargout = PsychTweak(cmd, varargin) %#ok<STOUT>
+function varargout = PsychTweak(cmd, varargin)
 % PsychTweak() - Apply tweaks to various Psychtoolbox parameters.
 %
 % This function allows to tweak some low-level operating parameters of
@@ -69,6 +69,27 @@ function varargout = PsychTweak(cmd, varargin) %#ok<STOUT>
 % existing directory where the .dot files should be written to.
 %
 %
+% PsychTweak('GStreamerPlaybackThreadingMethod', methodId);
+% -- Ask GStreamer to use multi-threaded video decoding method 'methodId'
+% for video playback if multi-threaded video decoding is requested by
+% user-code. Only some codecs support this. Currently available values are:
+% 1 = Frame-Threading, 2 = Slice-Threading, 3 = 2+1 = Frame and Slice
+% threading. By default, Frame+Slice threading will be used.
+%
+%
+% Debugging of USB based functions:
+% ---------------------------------
+%
+%
+% PsychTweak('LibUSBDebug', verbosity);
+%
+% -- Select level of verbosity for low-level debug output of USB functions.
+% This currently sets the debug level of libusb-1.0 based functions, e.g,
+% PsychHID, PsychKinectCore, some videocapture functions and others.
+% Possible values: 0 = Silence (default), 1 = Errors, 2 = Errors +
+% Warnings, 3 = Errors + Warnings + Info messages.
+%
+%
 % MS-Windows only tweaks:
 % -----------------------
 %
@@ -217,6 +238,20 @@ if strcmpi(cmd, 'ScreenVerbosity')
     return;
 end
 
+if strcmpi(cmd, 'LibUSBDebug')
+    if length(varargin) < 1
+        error('Must provide an initial LibUSBDebug level.');
+    end
+    
+    val = varargin{1};
+    if ~isnumeric(val) || ~isscalar(val)
+        error('Must provide a single integer as argument!');
+    end
+    
+    setenv('LIBUSB_DEBUG', sprintf('%i', round(val)));
+    return;
+end
+
 if strcmpi(cmd, 'GStreamerDebug')
     if length(varargin) < 1
         error('Must provide a GStreamer debug config string.');
@@ -249,6 +284,20 @@ if strcmpi(cmd, 'GStreamerDumpFilterGraph')
     return;
 end
 
+if strcmpi(cmd, 'GStreamerPlaybackThreadingMethod')
+    if length(varargin) < 1
+        error('Must provide an integer GStreamer playback threading method setting.');
+    end
+    
+    val = varargin{1};
+    if ~isscalar(val) || ~isnumeric(val)
+        error('Must provide an integer value as argument!');
+    end
+    
+    setenv('PSYCH_GST_THREAD_TYPES', sprintf('%i', floor(val)));
+    return;
+end
+
 if strcmpi(cmd, 'UseGPUIndex')
     if length(varargin) < 1
         error('Must provide a gpu index to use..');
