@@ -1,7 +1,7 @@
-function VideoCaptureDemo(fullscreen, fullsize, roi, depth, deviceId, cameraname)
+function VideoCaptureDemo(fullscreen, fullsize, roi, depth, deviceId, cameraname, bpc)
 % Demonstrate simple use of built-in video capture engine.
 %
-% VideoCaptureDemo([fullscreen=0][, fullsize=0][, roi=[0 0 640 480]][, depth][,deviceId=0][, cameraname])
+% VideoCaptureDemo([fullscreen=0][, fullsize=0][, roi=[0 0 640 480]][, depth][,deviceId=0][, cameraname][, bpc=8])
 %
 % VideoCaptureDemo initializes the first attached and supported camera on
 % your computer (e.g, the built-in iSight of Apple Macintosh computers),
@@ -36,6 +36,14 @@ function VideoCaptureDemo(fullscreen, fullsize, roi, depth, deviceId, cameraname
 % only honored if 'deviceId' is a negative number, and only for certain
 % video capture plugins. Defaults to none.
 %
+% 'bpc' Optional bit depth in bits per channel. Defaults to classic 8 bpc, but
+% some cameras support up to 16 bpc. Setting 16 bpc will try to coerce those into
+% providing "HDR" data. Usually this works with higher end firewire cameras and
+% the dc1394 capture engine. Your mileage with standard consumer cameras and the
+% default GStreamer capture engine will likely be less great. If at all, it would
+% probably only work on Linux or on OSX with GStreamer built from source, so you
+% have the camerabin1 plugin available.
+%
 
 % History:
 % Written sometimes 2006 by MK.
@@ -74,6 +82,11 @@ if nargin < 6
     cameraname = [];
 end
 
+if nargin < 7
+    % Default bpc to internal 8 bpc default:
+    bpc = [];
+end
+
 screenid=max(Screen('Screens'));
 
 try
@@ -82,14 +95,14 @@ try
     else
         win=Screen('OpenWindow', screenid, 0);
     end;
-
+    
     % Initial flip to a blank screen:
     Screen('Flip',win);
-
+    
     % Set text size for info text. 24 pixels is also good for Linux.
     Screen('TextSize', win, 24);
     
-    grabber = Screen('OpenVideoCapture', win, deviceId, roi, depth, [], [], cameraname);
+    grabber = Screen('OpenVideoCapture', win, deviceId, roi, depth, [], [], cameraname, [], [], bpc);
     %brightness = Screen('SetVideoCaptureParameter', grabber, 'Brightness',383)
     %exposure = Screen('SetVideoCaptureParameter', grabber, 'Exposure',130)
     %gain = Screen('SetVideoCaptureParameter', grabber, 'Gain')
@@ -100,75 +113,61 @@ try
     %model  = Screen('SetVideoCaptureParameter', grabber, 'GetModelname')
     %fps  = Screen('SetVideoCaptureParameter', grabber, 'GetFramerate')
     %roi  = Screen('SetVideoCaptureParameter', grabber, 'GetROI')
-
-for repcount=1:1
-    Screen('StartVideoCapture', grabber, 30, 1);
     
-    dstRect = [];
-    oldpts = 0;
-    count = 0;
-    t=GetSecs;
-    while (GetSecs - t) < 600 
-        if KbCheck
-            break;
-        end;
+    for repcount=1:1
+        Screen('StartVideoCapture', grabber, realmax, 1);
         
-        [tex pts nrdropped]=Screen('GetCapturedImage', win, grabber, 1); %#ok<NASGU>
-        % fprintf('tex = %i  pts = %f nrdropped = %i\n', tex, pts, nrdropped);
-        
-        if (tex>0)
-            % Perform first-time setup of transformations, if needed:
-            if fullsize & (count == 0) %#ok<AND2>
-                texrect = Screen('Rect', tex);
-                winrect = Screen('Rect', win);
-                sf = min([RectWidth(winrect) / RectWidth(texrect) , RectHeight(winrect) / RectHeight(texrect)]);
-                dstRect = CenterRect(ScaleRect(texrect, sf, sf) , winrect);
-            end
-
-            % Setup mirror transformation for horizontal flipping:
-            
-            % xc, yc is the geometric center of the text.
-            %[xc, yc] = RectCenter(Screen('Rect', win));
-            
-            % Make a backup copy of the current transformation matrix for later
-            % use/restoration of default state:
-            %Screen('glPushMatrix', win);
-            % Translate origin into the geometric center of text:
-            %Screen('glTranslate', win, xc, 0, 0);
-            % Apple a scaling transform which flips the direction of x-Axis,
-            % thereby mirroring the drawn text horizontally:
-            %Screen('glScale', win, -1, 1, 1);
-            % We need to undo the translations...
-            %Screen('glTranslate', win, -xc, 0, 0);
-            % The transformation is ready for mirrored drawing:
-
-            % Draw new texture from framegrabber.
-            Screen('DrawTexture', win, tex, [], dstRect);
-
-            %Screen('glPopMatrix', win);
-
-            % Print pts:
-            Screen('DrawText', win, sprintf('%.4f', pts - t), 0, 0, 255);
-            if count>0
-                % Compute delta:
-                delta = (pts - oldpts) * 1000;
-                oldpts = pts;
-                Screen('DrawText', win, sprintf('%.4f', delta), 0, 20, 255);
+        dstRect = [];
+        oldpts = 0;
+        count = 0;
+        t=GetSecs;
+        while (GetSecs - t) < 600
+            if KbCheck
+                break;
             end;
-
-            % Show it.
-            Screen('Flip', win);
-            Screen('Close', tex);
-            tex=0; %#ok<NASGU>
-        end;        
-        count = count + 1;
-    end;
-    telapsed = GetSecs - t %#ok<NOPRT>
-    Screen('StopVideoCapture', grabber);    
-end
+            
+            [tex pts nrdropped,intensity]=Screen('GetCapturedImage', win, grabber, 1); %#ok<ASGLU,NASGU>
+            % fprintf('tex = %i  pts = %f nrdropped = %i\n', tex, pts, nrdropped);
+            
+            if (tex>0)
+                % Perform first-time setup of transformations, if needed:
+                if fullsize && (count == 0)
+                    texrect = Screen('Rect', tex);
+                    winrect = Screen('Rect', win);
+                    sf = min([RectWidth(winrect) / RectWidth(texrect) , RectHeight(winrect) / RectHeight(texrect)]);
+                    dstRect = CenterRect(ScaleRect(texrect, sf, sf) , winrect);
+                end
+                
+                % texinfo = Screen('Getwindowinfo', tex)
+                % outintens = intensity
+                
+                % Draw new texture from framegrabber.
+                Screen('DrawTexture', win, tex, [], dstRect);
+                
+                %Screen('glPopMatrix', win);
+                
+                % Print pts:
+                Screen('DrawText', win, sprintf('%.4f', pts - t), 0, 0, 255);
+                if count>0
+                    % Compute delta:
+                    delta = (pts - oldpts) * 1000;
+                    oldpts = pts;
+                    Screen('DrawText', win, sprintf('%.4f', delta), 0, 20, 255);
+                end;
+                
+                % Show it.
+                Screen('Flip', win);
+                Screen('Close', tex);
+                tex=0; %#ok<NASGU>
+            end;
+            count = count + 1;
+        end;
+        telapsed = GetSecs - t %#ok<NOPRT>
+        Screen('StopVideoCapture', grabber);
+    end
     Screen('CloseVideoCapture', grabber);
     Screen('CloseAll');
     avgfps = count / telapsed %#ok<NOPRT,NASGU>
-catch
-   Screen('CloseAll');
+catch %#ok<CTCH>
+    Screen('CloseAll');
 end
