@@ -8,12 +8,14 @@ function CMClose(meterType)
 % meterType 3 is the CRS Colorimeter
 % meterType 4 is the PR655
 % meterType 5 is the PR670
+% meterType 6 is the PR705
 %
 % 2/15/02  dhb  Wrote it.
 % 4/13/02  dgp	Cosmetic.
 % 2/26/03  dhb  Added more meter types. 
 % 3/27/03  dhb, jmh  Fix up default argument.
 % 2/07/09  mk, tbc  Add PR-655 support.
+% 12/04/12 zlb  Add PR-705 support.
 
 % Set default meterType.
 if nargin < 1 || isempty(meterType)
@@ -37,6 +39,9 @@ switch meterType
     case 5
 		% PR-670
         PR670close;
+    case 6
+        % PR-705
+        PR705close;
 	otherwise,
 		error('Unknown meter type');
 end
