@@ -4,9 +4,10 @@ function PsychStartup
 % This performs setup of Matlab or Octave at session startup for
 % Psychtoolbox.
 %
-% On MS-Windows with 64-Bit Matlab it currently detects if the GStreamer
-% SDK runtime is installed, as this is required for Screen() to work. It
-% performs GStreamer setup, or outputs a warning if the runtime is missing.
+% On MS-Windows, it currently detects if the GStreamer SDK runtime is
+% installed, as this is required for Screen() multi-media functions to
+% work. It performs GStreamer setup, or outputs a warning if the runtime is
+% missing.
 %
 % This function is normally called from the startup.m Matlab startup file.
 %
@@ -14,24 +15,36 @@ function PsychStartup
 % History:
 % 14.09.2012  mk  Written.
 % 14.01.2013  mk  Make path detection more robust.
+% 12.09.2013  mk  Also apply GStreamer-SDK setup to 32-Bit Matlab on Windows.
 
 % Try-Catch protect the function, so Matlab startup won't fail due to
 % errors in this function:
 try
-    % Setup for 64-Bit Matlab on MS-Windows:
-    if IsWin(1) && ~IsOctave
+    % Setup for MS-Windows:
+    if IsWin
         % Need to assign a proper install location for the
         % www.gstreamer.com GStreamer-SDK runtime libraries, otherwise
-        % loading of the Screen() MEX file would fail due to unresolved
+        % use of GStreamer based functions would fail due to unresolved
         % link dependencies:
         
         % Find path to SDK-Root folder: Should be defined in env variable
         % by installer:
-        sdkroot = getenv('GSTREAMER_SDK_ROOT_X86_64');
+        if Is64Bit
+            sdkroot = getenv('GSTREAMER_SDK_ROOT_X86_64');
+            suffix = 'x86_64\';
+        else
+            sdkroot = getenv('GSTREAMER_SDK_ROOT_X86');
+            suffix = 'x86\';
+        end
+        
         if ~exist(sdkroot, 'dir')
             % Env variable points to non-existent SDK dir. How peculiar?
             % Invalidate invalid sdkroot, so fallback code can run:
-            fprintf('PsychStartup: Environment variable GSTREAMER_SDK_ROOT_X86_64 points to non-existent SDK folder?!?\n');
+            if Is64Bit
+                fprintf('PsychStartup: Environment variable GSTREAMER_SDK_ROOT_X86_64 points to non-existent SDK folder?!?\n');
+            else
+                fprintf('PsychStartup: Environment variable GSTREAMER_SDK_ROOT_X86 points to non-existent SDK folder?!?\n');
+            end
             fprintf('PsychStartup: The missing or inaccessible path to GStreamer is: %s\n', sdkroot);
             fprintf('PsychStartup: Something is botched. Trying various common locations for the SDK to keep going.\n');
             sdkroot = [];
@@ -39,46 +52,46 @@ try
 
         % Probe standard install location on drives C,D,E,F,G:
         if isempty(sdkroot) && exist('C:\gstreamer-sdk\0.10\', 'dir')
-            sdkroot = 'C:\gstreamer-sdk\0.10\x86_64\';
+            sdkroot = ['C:\gstreamer-sdk\0.10\' suffix];
         end
 
         if isempty(sdkroot) && exist('D:\gstreamer-sdk\0.10\', 'dir')
-            sdkroot = 'D:\gstreamer-sdk\0.10\x86_64\';
+            sdkroot = ['D:\gstreamer-sdk\0.10\' suffix];
         end
 
         if isempty(sdkroot) && exist('E:\gstreamer-sdk\0.10\', 'dir')
-            sdkroot = 'E:\gstreamer-sdk\0.10\x86_64\';
+            sdkroot = ['E:\gstreamer-sdk\0.10\' suffix];
         end
 
         if isempty(sdkroot) && exist('F:\gstreamer-sdk\0.10\', 'dir')
-            sdkroot = 'F:\gstreamer-sdk\0.10\x86_64\';
+            sdkroot = ['F:\gstreamer-sdk\0.10\' suffix];
         end
 
         if isempty(sdkroot) && exist('G:\gstreamer-sdk\0.10\', 'dir')
-            sdkroot = 'G:\gstreamer-sdk\0.10\x86_64\';
+            sdkroot = ['G:\gstreamer-sdk\0.10\' suffix];
         end
         
         % Probe install locations of legacy SDK's:
         if isempty(sdkroot) && exist('C:\gstreamer-sdk\2012.5\', 'dir')
-            sdkroot = 'C:\gstreamer-sdk\2012.5\x86_64\';
+            sdkroot = ['C:\gstreamer-sdk\2012.5\' suffix];
         end
         
         if isempty(sdkroot) && exist('C:\gstreamer-sdk\2012.7\', 'dir')
-            sdkroot = 'C:\gstreamer-sdk\2012.7\x86_64\';
+            sdkroot = ['C:\gstreamer-sdk\2012.7\' suffix];
         end
         
         if isempty(sdkroot) && exist('D:\gstreamer-sdk\2012.5\', 'dir')
-            sdkroot = 'D:\gstreamer-sdk\2012.5\x86_64\';
+            sdkroot = ['D:\gstreamer-sdk\2012.5\' suffix];
         end
         
         if isempty(sdkroot) && exist('D:\gstreamer-sdk\2012.7\', 'dir')
-            sdkroot = 'D:\gstreamer-sdk\2012.7\x86_64\';
+            sdkroot = ['D:\gstreamer-sdk\2012.7\' suffix];
         end
         
         if isempty(sdkroot)
             fprintf('\nPsychStartup: Path to GStreamer runtime is undefined! This probably means that\n');
-            fprintf('PsychStartup: the 64-Bit GStreamer SDK from www.gstreamer.com is not installed.\n');
-            fprintf('PsychStartup: The Psychtoolbox Screen() mex file will fail to work until you fix\n');
+            fprintf('PsychStartup: the 32-Bit or 64-Bit GStreamer SDK from www.gstreamer.com is not installed.\n');
+            fprintf('PsychStartup: The Psychtoolbox Screen() multimedia functions will fail to work until you fix\n');
             fprintf('PsychStartup: this! Read ''help GStreamer'' for instructions.\n\n');
         else
             sdkroot = [sdkroot 'bin'];
