@@ -78,6 +78,18 @@ winfo = Screen('GetWindowInfo', windowPtr);
 % Get current clut for use as backup copy later on:
 oldClut = Screen('ReadNormalizedGammaTable', windowPtr);
 
+% Try to use PsychGPUControl() method to disable display dithering
+% globally on all connected GPUs and displays. As of March 2013, this
+% function is only supported on Linux and Windows with AMD/ATI GPU's,
+% and only when running the proprietary Catalyst display driver.
+% It will no-op silently on other system configurations.
+%
+% We don't use the success status return code of the function, because
+% we don't know how trustworthy it is. Also this only affects dithering,
+% not gamma table identity setup, so the code-pathes below must run anyway
+% for proper setup, even if their dithering disable effect may be redundant.
+PsychGPUControl('SetDitheringEnabled', 0);
+
 % Ask Screen to use low-level setup code to configure the GPU for
 % untampered framebuffer pixel paththrough. This is only supported on a
 % subset of GPU's under certain conditions if the PsychtoolboxKernelDriver
@@ -139,7 +151,7 @@ else
         else
             % Full blown LUT given:
             lut = lutconfig;
-            if ~isnumeric(lut) || size(lut,1) < 1 || size(lut,2) ~= 3 %#ok<OR2>
+            if ~isnumeric(lut) || size(lut,1) < 1 || size(lut,2) ~= 3
                 sca;
                 error('LoadIdentityClut: Loaded data from config file is not a valid LUT! Not a numeric matrix or less than 1 row, or not 3 columns!');
             end
@@ -174,9 +186,9 @@ else
             end
             
             % Is it a Geforce-8000 or later (G80 core or later) and is this OS/X?
-            if ~isempty(strfind(winfo.GPUCoreId, 'G80')) && IsOSX %#ok<AND2>
+            if ~isempty(strfind(winfo.GPUCoreId, 'G80')) && IsOSX
                 % 10.5.x Leopard?
-                if (osxversion(1) == 10) && (osxversion(2) == 5) && (osxversion(3) >=0) %#ok<AND2>
+                if (osxversion(1) == 10) && (osxversion(2) == 5) && (osxversion(3) >=0)
                     % Yes. One of the releases with an embarassing amount of bugs,
                     % brought to you by Apple. Need to apply an especially ugly
                     % clut to force these cards into an identity mapping:
@@ -185,7 +197,7 @@ else
                 end
 
                 % 10.6.x Snow Leopard or later?
-                if (osxversion(1) == 10) && (osxversion(2) >= 6) && (osxversion(3) >=0) %#ok<AND2>
+                if (osxversion(1) == 10) && (osxversion(2) >= 6) && (osxversion(3) >=0)
                     % Yes. One of the releases with an embarassing amount of bugs,
                     % brought to you by Apple. Need to apply an especially ugly
                     % clut to force these cards into an identity mapping:
@@ -212,19 +224,19 @@ else
             end
         else
             if ~isempty(strfind(gfxhwtype, 'ATI')) || ~isempty(strfind(gfxhwtype, 'AMD')) || ~isempty(strfind(gfxhwtype, 'Advanced Micro Devices')) || ...
-                    ~isempty(strfind(winfo.GLRenderer, 'DRI R')) || ~isempty(strfind(winfo.GLRenderer, 'on ATI R')) %#ok<OR2>
+                    ~isempty(strfind(winfo.GLRenderer, 'DRI R')) || ~isempty(strfind(winfo.GLRenderer, 'on ATI R'))
                 % ATI card:
 
                 % A good default at least on OS/X is type 1:
                 gfxhwtype = 1;
 
-                if (IsWin || IsLinux) && ~isempty(strfind(winfo.GPUCoreId, 'R600')) %#ok<OR2,AND2>
+                if (IsWin || IsLinux) && ~isempty(strfind(winfo.GPUCoreId, 'R600'))
                     % At least the Radeon HD 3470 under Windows Vista and Linux needs type 0
                     % LUT's. Let's assume for the moment this is true for all R600
                     % cores, ie., all Radeon HD series cards.
                     fprintf('LoadIdentityClut: ATI Radeon HD-2000 or later detected. Enabling special type-0 LUT hacks for totally broken drivers.\n');
                     gfxhwtype = 0;
-                elseif (IsLinux) && (~isempty(strfind(winfo.GLRenderer, 'DRI R')) || ~isempty(strfind(winfo.GLRenderer, 'on ATI R'))) %#ok<OR2,AND2>
+                elseif (IsLinux) && (~isempty(strfind(winfo.GLRenderer, 'DRI R')) || ~isempty(strfind(winfo.GLRenderer, 'on ATI R')))
                     % At least the Radeon R3xx/4xx/5xx under Linux with DRI2 Mesa needs type 0
                     % LUT's. Let's assume for the moment this is true for all R600
                     % cores, ie., all Radeon HD series cards.
@@ -271,7 +283,7 @@ else
         oldClut = Screen('LoadNormalizedGammaTable', windowPtr, ((1/256:1/256:1)' * ones(1, 3)), loadOnNextFlip);
     end
 
-    if gfxhwtype == 2 && IsOSX %#ok<AND2>
+    if gfxhwtype == 2 && IsOSX
         % This works on OS/X 10.5 with NVidia GeForce 8800. It is an ugly hack
         % to compensate for the absolutely horrible, embarassing bugs in Apple's NVidia
         % graphics drivers and their clut handling. Does this company still
@@ -285,7 +297,7 @@ else
         oldClut = Screen('LoadNormalizedGammaTable', windowPtr, loadlut, loadOnNextFlip);
     end
 
-    if gfxhwtype == 3 && IsOSX %#ok<AND2>
+    if gfxhwtype == 3 && IsOSX
         % This works on OS/X 10.6.0 with NVidia Geforce-9200M according to CRS
         % and with 10.6.0 with Geforce-8800  in the MacPro according to
         % me. We assume this works on all G80 et al. GPU's:
@@ -300,7 +312,7 @@ else
         oldClut = Screen('LoadNormalizedGammaTable', windowPtr, loadlut, loadOnNextFlip);
     end
 
-    if gfxhwtype == 3 && IsWin %#ok<AND2>
+    if gfxhwtype == 3 && IsWin
         % This is an experimental variant of the OS/X type 3 lut, but with 256
         % slots. It is supposed for WindowsXP, assuming some NVidia GPU's,
         % e.g., some QuadroFX 3700 GPU's have similar problems:
@@ -315,7 +327,7 @@ else
         oldClut = Screen('LoadNormalizedGammaTable', windowPtr, loadlut, loadOnNextFlip);
     end
 
-    if gfxhwtype == 3 && IsLinux %#ok<AND2>
+    if gfxhwtype == 3 && IsLinux
         % NVidia QuadroFX cards with binary blob and lut's with more than
         % 256 slots. Upload a standard linear lut with matching number of
         % slots:
