@@ -54,6 +54,8 @@ function rc = PsychGPUControl(cmd, varargin)
 %                 blanks in path to executable.
 % 16.01.2011  mk  Add function to control desktop composition on Linux with
 %                 Compiz.
+% 18.04.2013  mk  Add use of 64-Bit ATIRadeonperf_Linux64 exe on 64-Bit Linux.
+%
 
 if nargin < 1
 	error('Subfunction command argument missing!');
@@ -165,7 +167,6 @@ return; %#ok<UNRCH>
 end
 
 function rc = executeRadeoncmd(cmdpostfix)
-    % Default to a return code of 1 for success:
     if IsOSX
         % A no-op on OS/X, as this is not supported at all.
         rc = 1;
@@ -173,10 +174,20 @@ function rc = executeRadeoncmd(cmdpostfix)
     end
 
     if IsLinux
+        % For 32-Bit Linux:
         cmdprefix = '/PsychContributed/ATIRadeonperf_Linux ';
     end
 
+    if IsLinux(1)
+        % For 64-Bit Linux: We have a dedicated 64-Bit executable, so we
+        % do not require installation of 32-Bit compatibility libraries on
+        % 64-Bit Linux systems just for our little exe here, as that would
+        % be wasteful:
+        cmdprefix = '/PsychContributed/ATIRadeonperf_Linux64 ';
+    end
+
     if IsWin
+        % For 32-Bit or 64-Bit Windows we always use a 32-Bit executable:
         cmdprefix = '/PsychContributed/ATIRadeonperf_Windows ';
     end
 
@@ -184,11 +195,18 @@ function rc = executeRadeoncmd(cmdpostfix)
     doCmd = strcat('"', [PsychtoolboxRoot cmdprefix] ,'"');
 
     % Call final command, return its return status code:
-    rc = system([doCmd cmdpostfix]);
+    [rc, msg] = system([doCmd cmdpostfix]);
 
     % Code has it backwards 1 = success, 0 = failure. Remap to our
     % convention:
     rc = 1 - rc;
     
+    % Output potential status or error messages, unless it is a message
+    % that signals we are not executing on a AMD/ATI GPU with Catalyst
+    % driver, ie., that the whole thing was a no-op:
+    if isempty(strfind(msg, 'ADL library not found!'))
+        disp(msg);
+    end
+
     return;
 end
