@@ -4,26 +4,32 @@ function UpdatePsychtoolbox(targetdirectory, targetRevision)
 % Update your working copy of the Psychtoolbox with the latest bug fixes,
 % enhancements, and features from the master server.
 %
+% If you are using a Psychtoolbox provided by NeuroDebian, this is not
+% needed. You will be automatically notified of updates to Psychtoolbox by
+% your operating systems update manager as soon as they become available.
+%
 % The "targetdirectory" argument is optional. If present, it gives the path
 % of the Psychtoolbox folder to update. If omitted, UpdatePsychtoolbox will
-% update the Psychtoolbox folder found by Matlab's or Octave's WHICH command. For
-% example:
+% update the Psychtoolbox folder found by Matlab's or Octave's "which"
+% command. For example:
+%
 % UpdatePsychtoolbox
 % UpdatePsychtoolbox('~/Applications/Psychtoobox')
 %
 % The "targetRevision" argument is optional and should be normally omitted.
 % Normal behaviour is to upgrade your working copy to the latest revision.
 % If you provide a specific targetRevision, then this script will
-% *downgrade* your copy of Psychtoolbox to the specified revision.
-% This is only useful if you experience problems after an update and want
-% to revert to an earlier known-to-be-good release.
-% Revisions can be specified by a revision number, a specific date, or by
-% the special flag 'PREV' which will downgrade to the revision before the
-% most current one. By executing this script multiple times with the 'PREV'
-% specifier, you can incrementally downgrade until stuff works for you.
+% *downgrade* your copy of Psychtoolbox to the specified revision. This is
+% only useful if you experience problems after an update and want to revert
+% to an earlier known-to-be-good release. Revisions can be specified by a
+% revision number, a specific date, or by the special flag 'PREV' which
+% will downgrade to the revision before the most current one. By executing
+% this script multiple times with the 'PREV' specifier, you can
+% incrementally downgrade until stuff works for you.
 %
 % UpdatePsychtoolbox cannot change the beta-vs-stable flavor of your
 % Psychtoolbox. To change the flavor, run DownloadPsychtoolbox again.
+%
 
 % History:
 %
@@ -52,6 +58,7 @@ function UpdatePsychtoolbox(targetdirectory, targetRevision)
 %
 % 09/14/12 mk  Drop support for Octave on MS-Windows.
 % 09/14/12 mk  Drop support for 32-Bit Octave on OSX.
+% 07/02/13 mk  Drop support for 32-Bit Matlab on OSX, and thereby for 32-Bit OSX.
 
 % Flush all MEX files: This is needed at least on M$-Windows for SVN to
 % work if Screen et al. are still loaded.
@@ -85,13 +92,14 @@ if any(isspace(targetdirectory))
     fprintf('The targetdirectory spec contains white-space. This should work, but has not been tested extensively.\n');
 end
 
-% Check if this is 32-Bit Octave on OSX, which we don't support anymore:
-if ~isempty(strfind(computer, 'apple-darwin')) && isempty(strfind(computer, '64'))
-    fprintf('Psychtoolbox 3.0.10 and later does no longer work with 32-Bit GNU/Octave on OSX.\n');
-    fprintf('You need to upgrade to a 64-Bit version of Octave on OSX, which is fully supported.\n');
+% Check if this is 32-Bit Octave or 32-Bit Matlab on OSX, which we don't support anymore:
+if (~isempty(strfind(computer, 'apple-darwin')) || strcmp(computer,'MACI')) && isempty(strfind(computer, '64'))
+    fprintf('Psychtoolbox 3.0.11 and later do no longer work with 32-Bit versions of Octave or Matlab on OSX.\n');
+    fprintf('You need to upgrade to a 64-Bit version of Octave or Matlab on OSX, which is fully supported.\n');
     fprintf('You can also use the alternate download function DownloadLegacyPsychtoolbox() to download\n');
-    fprintf('an old legacy copy of Psychtoolbox-3.0.9, which did support 32-Bit Octave 3.2 on OSX.\n');
-    error('Tried to setup on 32-Bit Octave, which is no longer supported on OSX.');
+    fprintf('an old legacy copy of Psychtoolbox-3.0.9, which did support 32-Bit Octave 3.2 on OSX, or use\n');
+    fprintf('DownloadPsychtoolbox() with flavor ''Psychtoolbox-3.0.10'', which does support 32-Bit Matlab on OSX.\n');
+    error('Tried to setup on 32-Bit Octave or 32-Bit Matlab, which are no longer supported on OSX.');
 end
 
 % Check if this is Octave on Windows, which we don't support at all:
